from .M60 import (
    BTMinerM60VK6A,
    BTMinerM60VK10,
    BTMinerM60VK20,
    BTMinerM60VK30,
    BTMinerM60VK40,
    BTMinerM60VL10,
    BTMinerM60VL20,
    BTMinerM60VL30,
    BTMinerM60VL40,
    BTMinerM60VL50,
)
from .M60S import (
    BTMinerM60SVK10,
    BTMinerM60SVK20,
    BTMinerM60SVK30,
    BTMinerM60SVK40,
    BTMinerM60SVL10,
    BTMinerM60SVL20,
    BTMinerM60SVL30,
    BTMinerM60SVL40,
    BTMinerM60SVL50,
    BTMinerM60SVL60,
    BTMinerM60SVL70,
)
from .M60S_Plus import (
    BTMinerM60SPlusVK30,
    BTMinerM60SPlusVK40,
    BTMinerM60SPlusVK50,
    BTMinerM60SPlusVK60,
    BTMinerM60SPlusVK70,
    BTMinerM60SPlusVL10,
    BTMinerM60SPlusVL30,
    BTMinerM60SPlusVL40,
    BTMinerM60SPlusVL50,
    BTMinerM60SPlusVL60,
)
from .M60S_Plus_Plus import BTMinerM60SPlusPlusVL30, BTMinerM60SPlusPlusVL40
from .M61 import (
    BTMinerM61VK10,
    BTMinerM61VK20,
    BTMinerM61VK30,
    BTMinerM61VK40,
    BTMinerM61VL10,
    BTMinerM61VL30,
    BTMinerM61VL40,
    BTMinerM61VL50,
    BTMinerM61VL60,
)
from .M61S import BTMinerM61SVL10, BTMinerM61SVL20, BTMinerM61SVL30
from .M61S_Plus import BTMinerM61SPlusVL30
from .M62S_Plus import BTMinerM62SPlusVK30
from .M63 import (
    BTMinerM63VK10,
    BTMinerM63VK20,
    BTMinerM63VK30,
    BTMinerM63VL10,
    BTMinerM63VL30,
)
from .M63S import (
    BTMinerM63SVK10,
    BTMinerM63SVK20,
    BTMinerM63SVK30,
    BTMinerM63SVK60,
    BTMinerM63SVL10,
    BTMinerM63SVL50,
    BTMinerM63SVL60,
)
from .M63S_Plus import (
    BTMinerM63SPlusVK30,
    BTMinerM63SPlusVL10,
    BTMinerM63SPlusVL20,
    BTMinerM63SPlusVL30,
    BTMinerM63SPlusVL50,
)
from .M63S_Plus_Plus import BTMinerM63SPlusPlusVL20
from .M64 import BTMinerM64VL30, BTMinerM64VL40
from .M64S import BTMinerM64SVL30
from .M65S import BTMinerM65SVK20, BTMinerM65SVL60
from .M65S_Plus import BTMinerM65SPlusVK30
from .M66 import BTMinerM66VK20, BTMinerM66VK30, BTMinerM66VL20, BTMinerM66VL30
from .M66S import (
    BTMinerM66SVK20,
    BTMinerM66SVK30,
    BTMinerM66SVK40,
    BTMinerM66SVK50,
    BTMinerM66SVK60,
    BTMinerM66SVL10,
    BTMinerM66SVL20,
    BTMinerM66SVL30,
    BTMinerM66SVL40,
    BTMinerM66SVL50,
)
from .M66S_Plus import (
    BTMinerM66SPlusVK30,
    BTMinerM66SPlusVL10,
    BTMinerM66SPlusVL20,
    BTMinerM66SPlusVL30,
    BTMinerM66SPlusVL40,
    BTMinerM66SPlusVL60,
)
from .M66S_Plus_Plus import BTMinerM66SPlusPlusVL20
from .M67S import BTMinerM67SVK30
