from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M56SPlusVJ30


class BTMinerM56SPlusVJ30(M5X, M56SPlusVJ30):
    pass


from pyasic.miners.device.models import M56SPlusVK30


class BTMinerM56SPlusVK30(M5X, M56SPlusVK30):
    pass


from pyasic.miners.device.models import M56SPlusVK40


class BTMinerM56SPlusVK40(M5X, M56SPlusVK40):
    pass


from pyasic.miners.device.models import M56SPlusVK50


class BTMinerM56SPlusVK50(M5X, M56SPlusVK50):
    pass
