from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M33V10


class BTMinerM33V10(M3X, M33V10):
    pass


from pyasic.miners.device.models import M33V20


class BTMinerM33V20(M3X, M33V20):
    pass


from pyasic.miners.device.models import M33V30


class BTMinerM33V30(M3X, M33V30):
    pass
