from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M53SVH20


class BTMinerM53SVH20(M5X, M53SVH20):
    pass


from pyasic.miners.device.models import M53SVH30


class BTMinerM53SVH30(M5X, M53SVH30):
    pass


from pyasic.miners.device.models import M53SVJ30


class BTMinerM53SVJ30(M5X, M53SVJ30):
    pass


from pyasic.miners.device.models import M53SVJ40


class BTMinerM53SVJ40(M5X, M53SVJ40):
    pass


from pyasic.miners.device.models import M53SVK30


class BTMinerM53SVK30(M5X, M53SVK30):
    pass
