from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M50SPlusPlusVK10


class BTMinerM50SPlusPlusVK10(M5X, M50SPlusPlusVK10):
    pass


from pyasic.miners.device.models import M50SPlusPlusVK20


class BTMinerM50SPlusPlusVK20(M5X, M50SPlusPlusVK20):
    pass


from pyasic.miners.device.models import M50SPlusPlusVK30


class BTMinerM50SPlusPlusVK30(M5X, M50SPlusPlusVK30):
    pass


from pyasic.miners.device.models import M50SPlusPlusVK40


class BTMinerM50SPlusPlusVK40(M5X, M50SPlusPlusVK40):
    pass


from pyasic.miners.device.models import M50SPlusPlusVK50


class BTMinerM50SPlusPlusVK50(M5X, M50SPlusPlusVK50):
    pass


from pyasic.miners.device.models import M50SPlusPlusVK60


class BTMinerM50SPlusPlusVK60(M5X, M50SPlusPlusVK60):
    pass


from pyasic.miners.device.models import M50SPlusPlusVL20


class BTMinerM50SPlusPlusVL20(M5X, M50SPlusPlusVL20):
    pass


from pyasic.miners.device.models import M50SPlusPlusVL30


class BTMinerM50SPlusPlusVL30(M5X, M50SPlusPlusVL30):
    pass


from pyasic.miners.device.models import M50SPlusPlusVL40


class BTMinerM50SPlusPlusVL40(M5X, M50SPlusPlusVL40):
    pass


from pyasic.miners.device.models import M50SPlusPlusVL50


class BTMinerM50SPlusPlusVL50(M5X, M50SPlusPlusVL50):
    pass


from pyasic.miners.device.models import M50SPlusPlusVL60


class BTMinerM50SPlusPlusVL60(M5X, M50SPlusPlusVL60):
    pass
