from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M31SV10


class BTMinerM31SV10(M3X, M31SV10):
    pass


from pyasic.miners.device.models import M31SV20


class BTMinerM31SV20(M3X, M31SV20):
    pass


from pyasic.miners.device.models import M31SV30


class BTMinerM31SV30(M3X, M31SV30):
    pass


from pyasic.miners.device.models import M31SV40


class BTMinerM31SV40(M3X, M31SV40):
    pass


from pyasic.miners.device.models import M31SV50


class BTMinerM31SV50(M3X, M31SV50):
    pass


from pyasic.miners.device.models import M31SV60


class BTMinerM31SV60(M3X, M31SV60):
    pass


from pyasic.miners.device.models import M31SV70


class BTMinerM31SV70(M3X, M31SV70):
    pass


from pyasic.miners.device.models import M31SV80


class BTMinerM31SV80(M3X, M31SV80):
    pass


from pyasic.miners.device.models import M31SV90


class BTMinerM31SV90(M3X, M31SV90):
    pass


from pyasic.miners.device.models import M31SVE10


class BTMinerM31SVE10(M3X, M31SVE10):
    pass


from pyasic.miners.device.models import M31SVE20


class BTMinerM31SVE20(M3X, M31SVE20):
    pass


from pyasic.miners.device.models import M31SVE30


class BTMinerM31SVE30(M3X, M31SVE30):
    pass
