from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M61SVL10


class BTMinerM61SVL10(M6X, M61SVL10):
    pass


from pyasic.miners.device.models import M61SVL20


class BTMinerM61SVL20(M6X, M61SVL20):
    pass


from pyasic.miners.device.models import M61SVL30


class BTMinerM61SVL30(M6X, M61SVL30):
    pass
