from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M33SPlusVG20


class BTMinerM33SPlusVG20(M3X, M33SPlusVG20):
    pass


from pyasic.miners.device.models import M33SPlusVG30


class BTMinerM33SPlusVG30(M3X, M33SPlusVG30):
    pass


from pyasic.miners.device.models import M33SPlusVH20


class BTMinerM33SPlusVH20(M3X, M33SPlusVH20):
    pass


from pyasic.miners.device.models import M33SPlusVH30


class BTMinerM33SPlusVH30(M3X, M33SPlusVH30):
    pass
