from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M60SPlusVK30


class BTMinerM60SPlusVK30(M6X, M60SPlusVK30):
    pass


from pyasic.miners.device.models import M60SPlusVK40


class BTMinerM60SPlusVK40(M6X, M60SPlusVK40):
    pass


from pyasic.miners.device.models import M60SPlusVK50


class BTMinerM60SPlusVK50(M6X, M60SPlusVK50):
    pass


from pyasic.miners.device.models import M60SPlusVK60


class BTMinerM60SPlusVK60(M6X, M60SPlusVK60):
    pass


from pyasic.miners.device.models import M60SPlusVK70


class BTMinerM60SPlusVK70(M6X, M60SPlusVK70):
    pass


from pyasic.miners.device.models import M60SPlusVL10


class BTMinerM60SPlusVL10(M6X, M60SPlusVL10):
    pass


from pyasic.miners.device.models import M60SPlusVL30


class BTMinerM60SPlusVL30(M6X, M60SPlusVL30):
    pass


from pyasic.miners.device.models import M60SPlusVL40


class BTMinerM60SPlusVL40(M6X, M60SPlusVL40):
    pass


from pyasic.miners.device.models import M60SPlusVL50


class BTMinerM60SPlusVL50(M6X, M60SPlusVL50):
    pass


from pyasic.miners.device.models import M60SPlusVL60


class BTMinerM60SPlusVL60(M6X, M60SPlusVL60):
    pass
