from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M31SEV10


class BTMinerM31SEV10(M3X, M31SEV10):
    pass


from pyasic.miners.device.models import M31SEV20


class BTMinerM31SEV20(M3X, M31SEV20):
    pass


from pyasic.miners.device.models import M31SEV30


class BTMinerM31SEV30(M3X, M31SEV30):
    pass
