from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M30SPlusV10


class BTMinerM30SPlusV10(M3X, M30SPlusV10):
    pass


from pyasic.miners.device.models import M30SPlusV100


class BTMinerM30SPlusV100(M3X, M30SPlusV100):
    pass


from pyasic.miners.device.models import M30SPlusV20


class BTMinerM30SPlusV20(M3X, M30SPlusV20):
    pass


from pyasic.miners.device.models import M30SPlusV30


class BTMinerM30SPlusV30(M3X, M30SPlusV30):
    pass


from pyasic.miners.device.models import M30SPlusV40


class BTMinerM30SPlusV40(M3X, M30SPlusV40):
    pass


from pyasic.miners.device.models import M30SPlusV50


class BTMinerM30SPlusV50(M3X, M30SPlusV50):
    pass


from pyasic.miners.device.models import M30SPlusV60


class BTMinerM30SPlusV60(M3X, M30SPlusV60):
    pass


from pyasic.miners.device.models import M30SPlusV70


class BTMinerM30SPlusV70(M3X, M30SPlusV70):
    pass


from pyasic.miners.device.models import M30SPlusV80


class BTMinerM30SPlusV80(M3X, M30SPlusV80):
    pass


from pyasic.miners.device.models import M30SPlusV90


class BTMinerM30SPlusV90(M3X, M30SPlusV90):
    pass


from pyasic.miners.device.models import M30SPlusVE100


class BTMinerM30SPlusVE100(M3X, M30SPlusVE100):
    pass


from pyasic.miners.device.models import M30SPlusVE30


class BTMinerM30SPlusVE30(M3X, M30SPlusVE30):
    pass


from pyasic.miners.device.models import M30SPlusVE40


class BTMinerM30SPlusVE40(M3X, M30SPlusVE40):
    pass


from pyasic.miners.device.models import M30SPlusVE50


class BTMinerM30SPlusVE50(M3X, M30SPlusVE50):
    pass


from pyasic.miners.device.models import M30SPlusVE60


class BTMinerM30SPlusVE60(M3X, M30SPlusVE60):
    pass


from pyasic.miners.device.models import M30SPlusVE70


class BTMinerM30SPlusVE70(M3X, M30SPlusVE70):
    pass


from pyasic.miners.device.models import M30SPlusVE80


class BTMinerM30SPlusVE80(M3X, M30SPlusVE80):
    pass


from pyasic.miners.device.models import M30SPlusVE90


class BTMinerM30SPlusVE90(M3X, M30SPlusVE90):
    pass


from pyasic.miners.device.models import M30SPlusVF20


class BTMinerM30SPlusVF20(M3X, M30SPlusVF20):
    pass


from pyasic.miners.device.models import M30SPlusVF30


class BTMinerM30SPlusVF30(M3X, M30SPlusVF30):
    pass


from pyasic.miners.device.models import M30SPlusVG20


class BTMinerM30SPlusVG20(M3X, M30SPlusVG20):
    pass


from pyasic.miners.device.models import M30SPlusVG30


class BTMinerM30SPlusVG30(M3X, M30SPlusVG30):
    pass


from pyasic.miners.device.models import M30SPlusVG40


class BTMinerM30SPlusVG40(M3X, M30SPlusVG40):
    pass


from pyasic.miners.device.models import M30SPlusVG50


class BTMinerM30SPlusVG50(M3X, M30SPlusVG50):
    pass


from pyasic.miners.device.models import M30SPlusVG60


class BTMinerM30SPlusVG60(M3X, M30SPlusVG60):
    pass


from pyasic.miners.device.models import M30SPlusVH10


class BTMinerM30SPlusVH10(M3X, M30SPlusVH10):
    pass


from pyasic.miners.device.models import M30SPlusVH20


class BTMinerM30SPlusVH20(M3X, M30SPlusVH20):
    pass


from pyasic.miners.device.models import M30SPlusVH30


class BTMinerM30SPlusVH30(M3X, M30SPlusVH30):
    pass


from pyasic.miners.device.models import M30SPlusVH40


class BTMinerM30SPlusVH40(M3X, M30SPlusVH40):
    pass


from pyasic.miners.device.models import M30SPlusVH50


class BTMinerM30SPlusVH50(M3X, M30SPlusVH50):
    pass


from pyasic.miners.device.models import M30SPlusVH60


class BTMinerM30SPlusVH60(M3X, M30SPlusVH60):
    pass


from pyasic.miners.device.models import M30SPlusVH70


class BTMinerM30SPlusVH70(M3X, M30SPlusVH70):
    pass


from pyasic.miners.device.models import M30SPlusVI30


class BTMinerM30SPlusVI30(M3X, M30SPlusVI30):
    pass


from pyasic.miners.device.models import M30SPlusVJ30


class BTMinerM30SPlusVJ30(M3X, M30SPlusVJ30):
    pass


from pyasic.miners.device.models import M30SPlusVJ40


class BTMinerM30SPlusVJ40(M3X, M30SPlusVJ40):
    pass
