from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M56SPlusPlusVK10


class BTMinerM56SPlusPlusVK10(M5X, M56SPlusPlusVK10):
    pass


from pyasic.miners.device.models import M56SPlusPlusVK30


class BTMinerM56SPlusPlusVK30(M5X, M56SPlusPlusVK30):
    pass


from pyasic.miners.device.models import M56SPlusPlusVK40


class BTMinerM56SPlusPlusVK40(M5X, M56SPlusPlusVK40):
    pass


from pyasic.miners.device.models import M56SPlusPlusVK50


class BTMinerM56SPlusPlusVK50(M5X, M56SPlusPlusVK50):
    pass
