from .M20 import BTMinerM20V10
from .M20P import BTMinerM20PV10, BTMinerM20PV30
from .M20S import BTMinerM20SV10, BTMinerM20SV20, BTMinerM20SV30
from .M20S_Plus import BTMinerM20SPlusV30
from .M21 import BTMinerM21V10
from .M21S import BTMinerM21SV20, BTMinerM21SV60, BTMinerM21SV70
from .M21S_Plus import BTMinerM21SPlusV20
from .M29 import BTMinerM29V10
