from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M63SVK10


class BTMinerM63SVK10(M6X, M63SVK10):
    pass


from pyasic.miners.device.models import M63SVK20


class BTMinerM63SVK20(M6X, M63SVK20):
    pass


from pyasic.miners.device.models import M63SVK30


class BTMinerM63SVK30(M6X, M63SVK30):
    pass


from pyasic.miners.device.models import M63SVK60


class BTMinerM63SVK60(M6X, M63SVK60):
    pass


from pyasic.miners.device.models import M63SVL10


class BTMinerM63SVL10(M6X, M63SVL10):
    pass


from pyasic.miners.device.models import M63SVL50


class BTMinerM63SVL50(M6X, M63SVL50):
    pass


from pyasic.miners.device.models import M63SVL60


class BTMinerM63SVL60(M6X, M63SVL60):
    pass
