from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M65SVK20


class BTMinerM65SVK20(M6X, M65SVK20):
    pass


from pyasic.miners.device.models import M65SVL60


class BTMinerM65SVL60(M6X, M65SVL60):
    pass
