from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M53SPlusPlusVK10


class BTMinerM53SPlusPlusVK10(M5X, M53SPlusPlusVK10):
    pass


from pyasic.miners.device.models import M53SPlusPlusVK20


class BTMinerM53SPlusPlusVK20(M5X, M53SPlusPlusVK20):
    pass


from pyasic.miners.device.models import M53SPlusPlusVK30


class BTMinerM53SPlusPlusVK30(M5X, M53SPlusPlusVK30):
    pass


from pyasic.miners.device.models import M53SPlusPlusVK50


class BTMinerM53SPlusPlusVK50(M5X, M53SPlusPlusVK50):
    pass


from pyasic.miners.device.models import M53SPlusPlusVL10


class BTMinerM53SPlusPlusVL10(M5X, M53SPlusPlusVL10):
    pass


from pyasic.miners.device.models import M53SPlusPlusVL30


class BTMinerM53SPlusPlusVL30(M5X, M53SPlusPlusVL30):
    pass
