from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M33SPlusPlusVG40


class BTMinerM33SPlusPlusVG40(M3X, M33SPlusPlusVG40):
    pass


from pyasic.miners.device.models import M33SPlusPlusVH20


class BTMinerM33SPlusPlusVH20(M3X, M33SPlusPlusVH20):
    pass


from pyasic.miners.device.models import M33SPlusPlusVH30


class BTMinerM33SPlusPlusVH30(M3X, M33SPlusPlusVH30):
    pass
