from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M30SV10


class BTMinerM30SV10(M3X, M30SV10):
    pass


from pyasic.miners.device.models import M30SV20


class BTMinerM30SV20(M3X, M30SV20):
    pass


from pyasic.miners.device.models import M30SV30


class BTMinerM30SV30(M3X, M30SV30):
    pass


from pyasic.miners.device.models import M30SV40


class BTMinerM30SV40(M3X, M30SV40):
    pass


from pyasic.miners.device.models import M30SV50


class BTMinerM30SV50(M3X, M30SV50):
    pass


from pyasic.miners.device.models import M30SV60


class BTMinerM30SV60(M3X, M30SV60):
    pass


from pyasic.miners.device.models import M30SV70


class BTMinerM30SV70(M3X, M30SV70):
    pass


from pyasic.miners.device.models import M30SV80


class BTMinerM30SV80(M3X, M30SV80):
    pass


from pyasic.miners.device.models import M30SVE10


class BTMinerM30SVE10(M3X, M30SVE10):
    pass


from pyasic.miners.device.models import M30SVE20


class BTMinerM30SVE20(M3X, M30SVE20):
    pass


from pyasic.miners.device.models import M30SVE30


class BTMinerM30SVE30(M3X, M30SVE30):
    pass


from pyasic.miners.device.models import M30SVE40


class BTMinerM30SVE40(M3X, M30SVE40):
    pass


from pyasic.miners.device.models import M30SVE50


class BTMinerM30SVE50(M3X, M30SVE50):
    pass


from pyasic.miners.device.models import M30SVE60


class BTMinerM30SVE60(M3X, M30SVE60):
    pass


from pyasic.miners.device.models import M30SVE70


class BTMinerM30SVE70(M3X, M30SVE70):
    pass


from pyasic.miners.device.models import M30SVF10


class BTMinerM30SVF10(M3X, M30SVF10):
    pass


from pyasic.miners.device.models import M30SVF20


class BTMinerM30SVF20(M3X, M30SVF20):
    pass


from pyasic.miners.device.models import M30SVF30


class BTMinerM30SVF30(M3X, M30SVF30):
    pass


from pyasic.miners.device.models import M30SVG10


class BTMinerM30SVG10(M3X, M30SVG10):
    pass


from pyasic.miners.device.models import M30SVG20


class BTMinerM30SVG20(M3X, M30SVG20):
    pass


from pyasic.miners.device.models import M30SVG30


class BTMinerM30SVG30(M3X, M30SVG30):
    pass


from pyasic.miners.device.models import M30SVG40


class BTMinerM30SVG40(M3X, M30SVG40):
    pass


from pyasic.miners.device.models import M30SVH10


class BTMinerM30SVH10(M3X, M30SVH10):
    pass


from pyasic.miners.device.models import M30SVH20


class BTMinerM30SVH20(M3X, M30SVH20):
    pass


from pyasic.miners.device.models import M30SVH30


class BTMinerM30SVH30(M3X, M30SVH30):
    pass


from pyasic.miners.device.models import M30SVH40


class BTMinerM30SVH40(M3X, M30SVH40):
    pass


from pyasic.miners.device.models import M30SVH50


class BTMinerM30SVH50(M3X, M30SVH50):
    pass


from pyasic.miners.device.models import M30SVH60


class BTMinerM30SVH60(M3X, M30SVH60):
    pass


from pyasic.miners.device.models import M30SVI20


class BTMinerM30SVI20(M3X, M30SVI20):
    pass


from pyasic.miners.device.models import M30SVJ30


class BTMinerM30SVJ30(M3X, M30SVJ30):
    pass
